import 'dart:io';

import 'package:dart_ng/filename.dart';

// AngularDart Helper
// Works like "ng create"
// Usage: create <component|pipe|service> <name>

const String C_HTML = """<h1>{{TemplateName}}</h1>
""";
const String C_SCSS = """:host {
    /* This is equivalent of the 'body' selector of a page. */
}
""";
const String C_DART = """import 'package:angular/angular.dart';
import 'package:angular_components/angular_components.dart';

@Component(
  selector: '{{templateName}}',
  styleUrls: const ['{{templateName}}.css'],
  templateUrl: '{{templateName}}.html',
  directives: const [
    materialDirectives
  ],
  providers: const [
    materialProviders
  ],
)
class {{TemplateName}}Component implements OnInit {
  @override
  ngOnInit() async {
  }
}
""";
const String P_DART = """import 'package:angular/angular.dart';

@Pipe('{{TemplateName}}')
class {{TemplateName}}Pipe implements PipeTransform {
  String transform(value) {
    return("TODO");
  }
}
""";
const String S_DART = """import 'dart:async';
import 'dart:convert';

import 'package:angular/core.dart';
import 'package:http/browser_client.dart';

@Injectable()
class {{TemplateName}}Service {
  final BrowserClient _http;
  
  {{TemplateName}}Service(this._http);

  /* GET/POST Examples
  **
  Future<dynamic> getAPI() async {
    final response = await _http.get("/api");
    return(JSON.decode(response.body));
  }
  
  Future<dynamic> postData(data) async {
    final response = await _http.post("/api", headers: {'Content-type': 'application/json'}, body: JSON.encode(data));
    // or map list to a class with :
    // return (JSON.decode(response.body).map((json) => new MyClass.fromJson(json)).toList());
    return(JSON.decode(response.body));
  }
  **
  */
}
""";

void usage() {
  print("Usage: create <component|pipe|service> <name>");
  exit(5);
}

int main(List<String> args) {
  if (args.length != 2) {
    usage();
  }

  try {
    FileName name = new FileName(args[1]);
    print(name.toString());
    switch (args[0]) {
      case "component":
        createComponent(name);
        break;
      case "pipe":
        createPipe(name);
        break;
      case "service":
        createService(name);
        break;

      default:
        usage();
        break;
    }
  } catch (e) {
    print("Error : " + e.message);
    return (10);
  }
  return (0);
}

String getTemplate(String tpl, String u, String l) {
  return (tpl.replaceAll("{{TemplateName}}", u).replaceAll("{{templateName}}", l));
}

void writeTemplate(Directory dir, String tpl, FileName name, String ext) {
  writeTemplateFile(new File(dir.path + Platform.pathSeparator + name.lname + ext), tpl, name);
}

void writeTemplateFile(File file, String tpl, FileName name) {
  print("Writing template : ${file.path}");
  file.writeAsStringSync(getTemplate(tpl, name.uname, name.lname));
}

void createComponent(FileName name) {
  Directory dir = new Directory("lib" + Platform.pathSeparator + name.lname);

  if (dir.existsSync()) {
    throw new Exception("Directory \"${dir.path}\" already exists !");
  }

  print("Create directory : ${name.lname}");
  dir.createSync();

  writeTemplate(dir, C_HTML, name, ".html");
  writeTemplate(dir, C_DART, name, ".dart");
  writeTemplate(dir, C_SCSS, name, ".scss");
}

void createPipe(FileName name) {
  File pipe = new File("lib" + Platform.pathSeparator + name.lname + "_pipe.dart");

  if (pipe.existsSync()) {
    throw new Exception("File \"${pipe.path}\" already exists !");
  }
  writeTemplateFile(pipe, P_DART, name);
}

void createService(FileName name) {
  File service = new File("lib" + Platform.pathSeparator + name.lname + "_service.dart");

  if (service.existsSync()) {
    throw new Exception("File \"${service.path}\" already exists !");
  }
  writeTemplateFile(service, S_DART, name);
}